
/*---------------------------------------------------------------------------

    Filename: fieldpicker.js
    Project:  Controls
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:        Control for picking multiple fields from a table

    Developer(s):   David Z. Williams

---------------------------------------------------------------------------*/


class FieldPicker extends Form
{
    function FieldPicker()
    {
        super("FieldPicker", 0, 0, 400, 280);
        
        // array of custom fields to add to the source listview
        m_custom_fields = new Array;
        
        // create the fields listview
        m_fields_listview = new ListView;
        m_fields_listview.setView(ListView.Details);
        m_fields_listview.addColumn("Available Fields", 180, ListView.AlignLeft);
        m_fields_listview.addItem(new ListViewItem("All (0 Fields)", getBlankBitmap()));
        
        // create the selected fields listview
        m_selected_fields_listview = new ListView;
        m_selected_fields_listview.setView(ListView.Details);
        m_selected_fields_listview.addColumn("Fields", 180, ListView.AlignLeft);
        
        // -- create add/remove buttons --
        
        var addfield_button = new BitmapButton(getAddFieldBitmap());
        addfield_button.click.connect(this, onAddFieldClicked);
        
        var removefield_button = new BitmapButton(getRemoveFieldBitmap());
        removefield_button.click.connect(this, onRemoveFieldClicked);
        
        // create the add field/remove field button layout
        var addremove_button_layout = new BoxLayout(Layout.Vertical);
        addremove_button_layout.addStretchSpacer();
        addremove_button_layout.add(addfield_button);
        addremove_button_layout.addSpacer(8);
        addremove_button_layout.add(removefield_button);
        addremove_button_layout.addStretchSpacer();
        
        // -- create add/remove buttons --
        
        var moveup_button = new BitmapButton(getMoveUpBitmap());
        moveup_button.click.connect(this, onMoveUpClicked);
        
        var movedown_button = new BitmapButton(getMoveDownBitmap());
        movedown_button.click.connect(this, onMoveDownClicked);
        
        // create the move up/move down button layout
        m_updown_button_layout = new BoxLayout(Layout.Vertical);
        m_updown_button_layout.addStretchSpacer();
        m_updown_button_layout.add(moveup_button);
        m_updown_button_layout.addSpacer(8);
        m_updown_button_layout.add(movedown_button);
        m_updown_button_layout.addStretchSpacer();
        
        // create the main layout
        m_main_layout = new BoxLayout(Layout.Horizontal);
        m_main_layout.add(m_fields_listview, 1, Layout.Expand);
        m_main_layout.addSpacer(8);
        m_main_layout.add(addremove_button_layout, 0, Layout.Expand);
        m_main_layout.addSpacer(8);
        m_main_layout.add(m_selected_fields_listview, 1, Layout.Expand);
        m_main_layout.add(m_updown_button_layout, 0, Layout.Expand | Layout.Left, 8);
        setLayout(m_main_layout);

        m_main_layout.show(m_updown_button_layout, false);
        layout();
    }
    
    // -- functions on the "public" interface --
    
    function setTable(path)
    {
        // find out if the path contains a valid table
        var db = HostApp.getDatabase();
        if (!db.describeTable(path))
            return;
        
        // we've already set this table as our table
        if (m_table_path == path)
            return;
        
        m_table_path = path;
        refresh();
    }
    
    function getTable()
    {
        return m_table_path;
    }
    
    function getSelectedFields()
    {
        var retval = new Array;
        
        var items = m_selected_fields_listview.getItems();
        for (var i in items)
            retval.push(items[i].getColumnText(0));
        
        return retval;
    }
    
    function addCustomField(name)
    {
        m_custom_fields.push(name);
    }
    
    function clearCustomFields()
    {
        m_custom_fields.clear();
    }
    
    function addSelectedField(name)
    {
        m_selected_fields_listview.addItem(new ListViewItem(name, getFieldBitmap()));
    }
    
    function clearSelectedFields()
    {
        m_selected_fields_listview.clear();
    }
    
    /*
    // TODO: Until we have a means of "reselecting" listview items,
    //       the up/down buttons don't really work properly
    function showUpDownButtons(show)
    {
        if (show == null)
            show = true;
        m_main_layout.show(m_updown_button_layout, show);
        layout();
    }
    */
    
    function refresh()
    {
        populateFieldsList();
    }
    
    // -- "private" functions --
    
    function populateFieldsList()
    {
        // freeze
        disablePaint();
        
        // clear the list and add 'All Fields' item
        m_fields_listview.clear();
        m_fields_listview.addItem(new ListViewItem("All (0 Fields)", getBlankBitmap()));
        
        // add custom fields to the list
        for (var i in m_custom_fields)
            m_fields_listview.addItem(new ListViewItem(m_custom_fields[i], getFieldBitmap()));
        
        // add fields to the list
        var db = HostApp.getDatabase();
        var fields = db.describeTable(m_table_path);
        for (var j in fields)
            m_fields_listview.addItem(new ListViewItem(fields[j].name, getFieldBitmap()));
        
        // update the 'All Fields' item label
        updateAllFieldsLabel();
        
        // thaw
        enablePaint();
    }
    
    function updateAllFieldsLabel()
    {
        var field_count = m_fields_listview.getItemCount()-1;
        if (field_count == 1)
            m_fields_listview.getItem(0).setColumnText(0, "All (1 Field)");
             else
            m_fields_listview.getItem(0).setColumnText(0, "All (" + field_count + " Fields)");
    }
    
    // -- "private" event handlers --
    
    function onAddFieldClicked()
    {
        // find out if the 'All Fields' item is selected
        var item = m_fields_listview.getSelectedItem();
        var all_items = (item && item.getIndex() == 0) ? true : false;
        
        var items;
        if (all_items)
            items = m_fields_listview.getItems();
             else
            items = m_fields_listview.getSelectedItems();
        
        for (var i in items)
        {
            // don't include the 'All Fields' item
            if (items[i].getIndex() == 0)
                continue;
            
            m_selected_fields_listview.addItem(items[i]);
        }
    }
    
    function onRemoveFieldClicked()
    {
        var items = m_selected_fields_listview.getSelectedItems();
        for (var i in items)
            m_selected_fields_listview.deleteItem(items[i]);
    }
    
    function onMoveUpClicked()
    {
        var selected = m_selected_fields_listview.getSelectedItems();
        if (selected.length == 0)
            return;
        
        // freeze
        disablePaint();
        
        // clear the selected fields listview
        var all = m_selected_fields_listview.getItems();
        m_selected_fields_listview.clear();
        
        // -- mark selected items --
        for (var i in all)
        {
            all[i].selected = false;
            
            for (var j in selected)
            {
                if (all[i] == selected[j])
                {
                    all[i].selected = true;
                    break;
                }
            }
        }
        
        // -- reorder items in the result array --
        var item;
        var res = new Array;
        for (var i in all)
        {
            // the result array is empty; add a starter item
            if (res.length == 0)
            {
                res.push(all[i]);
                continue;
            }
            
            // add unselected items to the result array
            if (!all[i].selected)
            {
                res.push(all[i]);
                continue;
            }
            
            if (all[i].selected)
            {
                // remove the last item from the result array
                item = res.pop();
                
                // the previous item was selected;
                // add it back before this selected item
                if (item.selected)
                    res.push(item);
                
                // add the selected item to the array
                res.push(all[i]);
                
                // the previous item was not selected;
                // add it back after this selected item
                if (!item.selected)
                    res.push(item);
            }
        }
        
        // repopulate the selected fields listview
        for (var i in res)
            m_selected_fields_listview.addItem(res[i]);
        
        // thaw
        enablePaint();
    }
    
    function onMoveDownClicked()
    {
        var selected = m_selected_fields_listview.getSelectedItems();
        if (selected.length == 0)
            return;
        
        // freeze
        disablePaint();
        
        // clear the selected fields listview
        var all = m_selected_fields_listview.getItems();
        m_selected_fields_listview.clear();
        
        // -- mark selected items --
        for (var i in all)
        {
            all[i].selected = false;
            
            for (var j in selected)
            {
                if (all[i] == selected[j])
                {
                    all[i].selected = true;
                    break;
                }
            }
        }
        
        // -- reorder items in the result array --
        var item;
        var to_add = new Array;
        var res = new Array;
        for (var i in all)
        {
            // add unselected items to the result array
            if (!all[i].selected)
                res.push(all[i]);
            
            // keep track of the selected items
            if (all[i].selected)
                to_add.push(all[i]);
            
            // now that we've added the unselected item below the selected
            // item(s), we can add the selected item(s) back
            if (!all[i].selected && to_add.length > 0)
            {
                while(to_add.length > 0)
                    res.push(to_add.shift());
            }
        }
        
        // add any "straggler" items to the result array
        if (to_add.length > 0)
        {
            while(to_add.length > 0)
                res.push(to_add.shift());
        }
        
        // repopulate the selected fields listview
        for (var i in res)
            m_selected_fields_listview.addItem(res[i]);
        
        // thaw
        enablePaint();
    }
    
    // controls and layouts
    var m_fields_listview;
    var m_selected_fields_listview;
    var m_updown_button_layout;
    var m_main_layout;
    
    // custom fields array
    var m_custom_fields;
    
    // source table path
    var m_table_path;
}




var g_empty_bitmap_16;
var g_field_bitmap_16;
var g_moveup_bitmap_16;
var g_movedown_bitmap_16;
var g_addfield_bitmap_16;
var g_removefield_bitmap_16;

function getBlankBitmap()
{
    if (g_empty_bitmap_16)
        return g_empty_bitmap_16;
    
    g_empty_bitmap_16 = new Bitmap('
        "16 16 1 1",
        "  c None",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                "
                          ');
    
    return g_empty_bitmap_16;
}

function getFieldBitmap()
{
    if (g_field_bitmap_16)
        return g_field_bitmap_16;
    
    g_field_bitmap_16 = new Bitmap('
        "16 16 20 1",
        "* c #4CAEE5",
        ". c #D0E1F2",
        "; c #3F78BF",
        "$ c #2A7EC7",
        "1 c #CACCD0",
        ": c #F2F5F9",
        "# c #4FB7E8",
        "a c #429CDE",
        "  c None",
        "> c #3969AE",
        "X c #2C86CB",
        "@ c #53BFEA",
        "= c #4BABE3",
        "o c #3592D3",
        "< c #E6E6E6",
        "+ c #D0DFF0",
        "O c #5ACBEE",
        "- c #ECEEF2",
        "% c #5082C2",
        "& c #46A6E1",
        "                ",
        "                ",
        "      .X.       ",
        "     .oOo.      ",
        "    .oOOOo.     ",
        "   +oOOOOOX+    ",
        "  +X@@@@@@@X+   ",
        " +X#########$+  ",
        " %&******===#%  ",
        " -;&&&&&&&&*;:  ",
        "  :>aaaaaa*;:   ",
        "   :>aaoa=;:    ",
        "    :>oa=;:     ",
        "     <>o>-      ",
        "    :<1>1<-:    ",
        "                "
                          ');
    
    return g_field_bitmap_16;
}

function getMoveUpBitmap()
{
    if (g_moveup_bitmap_16)
        return g_moveup_bitmap_16;
    
    g_moveup_bitmap_16 = new Bitmap('
        "16 16 2 1",
        "  c None",
        ". c #000000",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "       .        ",
        "      ...       ",
        "     .....      ",
        "    .......     ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                "
                          ');
    
    return g_moveup_bitmap_16;
}

function getMoveDownBitmap()
{
    if (g_movedown_bitmap_16)
        return g_movedown_bitmap_16;
    
    g_movedown_bitmap_16 = new Bitmap('
        "16 16 2 1",
        "  c None",
        ". c #000000",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "    .......     ",
        "     .....      ",
        "      ...       ",
        "       .        ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                "
                          ');
    
    return g_movedown_bitmap_16;
}

function getAddFieldBitmap()
{
    if (g_addfield_bitmap_16)
        return g_addfield_bitmap_16;
    
    g_addfield_bitmap_16 = new Bitmap('
        "16 16 2 1",
        "  c None",
        ". c #000000",
        "                ",
        "                ",
        "                ",
        "                ",
        "      .         ",
        "      ..        ",
        "      ...       ",
        "      ....      ",
        "      ...       ",
        "      ..        ",
        "      .         ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                "
                          ');
    
    return g_addfield_bitmap_16;
}

function getRemoveFieldBitmap()
{
    if (g_removefield_bitmap_16)
        return g_removefield_bitmap_16;
    
    g_removefield_bitmap_16 = new Bitmap('
        "16 16 2 1",
        "  c None",
        ". c #000000",
        "                ",
        "                ",
        "                ",
        "                ",
        "         .      ",
        "        ..      ",
        "       ...      ",
        "      ....      ",
        "       ...      ",
        "        ..      ",
        "         .      ",
        "                ",
        "                ",
        "                ",
        "                ",
        "                "
                          ');
    
    return g_removefield_bitmap_16;
}




